/*
** AUTHORS:
** Rama Alebouyeh (rama[at]cs.ucsb.edu)
** Matthew S Allen (msa[at]cs.ucsb.edu)
*/
#ifndef _HOST_H_
#define _HOST_H_

#include "key.h"

typedef struct {
  char *name;
  unsigned long address;
  int failed;
  double failuretime;
  int port;
  double latency;
  double loss;
  double success;
  Key key;
} ChimeraHost;

typedef struct { 
  void * network;
  void * message;
  void * route;
  void * log;
  void * host;
  void * chimera;
} ChimeraState;

/** host_get: 
 ** gets a host entry for the given host, getting it from the cache if
 ** possible, or alocates memory for it
 */
ChimeraHost *host_get(ChimeraState *state,char *hn, int port);

/** host_release:
 ** releases a host from the cache, declaring that the memory could be
 ** freed any time. returns NULL if the entry is deleted, otherwise it
 ** returns #host#
 */
void host_release(ChimeraState *state,ChimeraHost *host);

/** host_decode: 
 ** decodes a string into a chimera host structure. This acts as a
 ** host_get, and should be followed eventually by a host_release. 
 */
ChimeraHost *host_decode(ChimeraState *state,char *s);

/** host_encode:
 ** encodes the #host# into a string, putting it in #s#, which has
 ** #len# bytes in it.
 */
void host_encode(char *s, int len, ChimeraHost *host);

/** host_init: 
 ** initialize a host struct with a #size# element cache.
 */
void* host_init(int size);

#endif /* _HOST_H_ */
